import{f as c,a as d}from"../chunks/C0Fr8dve.js";import{p as m,o as f,t as u,i as p,j as s,k as o,g as v,l as _}from"../chunks/NqQ1dWOy.js";import{s as h,a as r,b,t as g}from"../chunks/DR8nis88.js";const k=!0,x=!1,O=Object.freeze(Object.defineProperty({__proto__:null,prerender:k,ssr:x},Symbol.toStringTag,{value:"Module"}));var y=c('<div><main class="flex-1 overflow-hidden"><!></main></div>');function w(n,t){m(t,!0),f(()=>(r.connect(),()=>{r.disconnect()}));let l=_(()=>g.current==="dark"?"dark":"");var e=y(),a=s(e),i=s(a);h(i,t,"default",{},null),o(a),o(e),u(()=>b(e,1,`${v(l)??""} min-h-screen bg-slate-50 dark:bg-slate-900 flex flex-col transition-colors`)),d(n,e),p()}export{w as component,O as universal};
